@@ -21,24 +21,31 @@ import logging
 import typing
 import os
 import sys
-from shutil import get_terminal_size
 import json
 from functools import wraps
 from typing import Tuple
 from typing import Optional
+from typing import Set
+from typing import Dict, Any
 
 import yaml
-from texttable import Texttable
 import click
-from termcolor import colored
+from rich.console import Console
+from rich.text import Text
+from rich.table import Table
+from rich import box
 import daiquiri
 from thoth.python import Project
+from thoth.common import cwd
 from thoth.common import ThothAdviserIntegrationEnum
+from thoth.common import get_justification_link as jl
 from thamos.exceptions import NoProjectDirError
+from thamos.exceptions import NoRuntimeEnvironmentError
 from thamos.config import config as configuration
-from thamos.lib import advise as thoth_advise
+from thamos.lib import advise_here as thoth_advise_here
 from thamos.lib import provenance_check as thoth_provenance_check
 from thamos.lib import get_log
+from thamos.lib import install as thamos_install
 from thamos.lib import get_status
 from thamos.utils import workdir
 from thamos import __version__ as thamos_version
@@ -49,11 +56,11 @@ daiquiri.setup(level=logging.INFO)
 _LOGGER = logging.getLogger("thamos")
 
 _EMOJI = {
-    "WARNING": colored("\u26a0\ufe0f WARNING", "yellow"),
-    "ERROR": colored("\u274c ERROR", "red", attrs=["bold"]),
-    "INFO": colored("\u2714\ufe0f INFO", "green"),
-    "LATEST": colored("\U0001f44c LATEST", "green"),
-    "CVE": colored("\u2620\uFE0F  CVE \u2620\uFE0F", "red"),
+    "WARNING": Text("\u26a0\ufe0f WARNING", style="yellow"),
+    "ERROR": Text("\u274c ERROR", style="bold red"),
+    "INFO": Text("\u2714\ufe0f INFO", "green"),
+    "LATEST": Text("\U0001f44c LATEST", "cyan"),
+    "CVE": Text("\u2620\uFE0F  CVE \u2620\uFE0F", "red"),
 }
 
 # Align of columns in table - default is left, values stated here are adjusted otherwise.
@@ -67,6 +74,7 @@ _TABLE_COLS_ALIGN = {
 
 def handle_cli_exception(func: typing.Callable) -> typing.Callable:
     """Suppress exception in CLI if debug mode was not turned on."""
+    # noqa
     @wraps(func)
     def wrapper(ctx, *args, **kwargs):
         try:
@@ -84,15 +92,48 @@ def handle_cli_exception(func: typing.Callable) -> typing.Callable:
 def _load_files(requirements_format: str) -> Tuple[str, Optional[str]]:
     """Load Pipfile/Pipfile.lock or requirements.in/txt from the current directory."""
     if requirements_format == "pipenv":
-        project = Project.from_files(without_pipfile_lock=not os.path.exists("Pipfile.lock"))
+        _LOGGER.info("Using Pipenv files located in %r directory", os.getcwd())
+        pipfile_lock_exists = os.path.exists("Pipfile.lock")
+
+        if pipfile_lock_exists:
+            _LOGGER.info(
+                "Submitting Pipfile.lock as a base for user's stack scoring - see %s",
+                jl("user_stack"),
+            )
+
+        project = Project.from_files(
+            without_pipfile_lock=not os.path.exists("Pipfile.lock")
+        )
+
+        if (
+            pipfile_lock_exists
+            and project.pipfile_lock.meta.hash["sha256"]
+            != project.pipfile.hash()["sha256"]
+        ):
+            _LOGGER.error(
+                "Pipfile hash stated in Pipfile.lock %r does not correspond to Pipfile hash %r - was Pipfile "
+                "adjusted? This error is not critical.",
+                project.pipfile_lock.meta.hash["sha256"][:6],
+                project.pipfile.hash()["sha256"][:6],
+            )
     elif requirements_format in ("pip", "pip-tools", "pip-compile"):
+        _LOGGER.info("Using requirements.txt file located in %r directory", os.getcwd())
         project = Project.from_pip_compile_files(allow_without_lock=True)
     else:
-        raise ValueError(f"Unknown configuration option for requirements format: {requirements_format!r}")
-    return project.pipfile.to_string(), project.pipfile_lock.to_string() if project.pipfile_lock else None
+        raise ValueError(
+            f"Unknown configuration option for requirements format: {requirements_format!r}"
+        )
+    return (
+        project.pipfile.to_string(),
+        project.pipfile_lock.to_string() if project.pipfile_lock else None,
+    )
 
 
-def _write_files(requirements: str, requirements_lock: str, requirements_format: str) -> None:
+def _write_files(
+    requirements: Dict[str, Any],
+    requirements_lock: Dict[str, Any],
+    requirements_format: str,
+) -> None:
     """Write content of Pipfile/Pipfile.lock or requirements.in/txt to the current directory."""
     project = Project.from_dict(requirements, requirements_lock)
     if requirements_format == "pipenv":
@@ -108,18 +149,9 @@ def _write_files(requirements: str, requirements_lock: str, requirements_format:
         )
 
 
-def _print_header(header: str) -> None:
-    """Print header to terminal respecting terminal size."""
-    terminal_size = get_terminal_size()
-    padding = (terminal_size.columns - len(header) - 2) // 2
-    click.echo(padding * " " + header)
-    click.echo(padding * " " + "=" * len(header) + "  \n")
-
-
 def _write_configuration(
     advised_configuration: dict,
     recommendation_type: str = None,
-    limit_latest_versions: int = None,
     dev: bool = False,
 ) -> None:
     """Create thoth configuration file."""
@@ -148,10 +180,6 @@ def _write_configuration(
             runtime_environment_entry = advised_configuration
             if recommendation_type:
                 runtime_environment_entry["recommendation_type"] = recommendation_type
-            if limit_latest_versions:
-                runtime_environment_entry[
-                    "limit_latest_versions"
-                ] = limit_latest_versions
             runtime_environment_entry["dev"] = dev
             content["runtime_environments"][idx] = runtime_environment_entry
             break
@@ -167,17 +195,22 @@ def _write_configuration(
         yaml.safe_dump(content, thoth_yaml_file)
 
 
-def _print_report(report: dict, json_output: bool = False):
+def _print_report(report: dict, json_output: bool = False, title: Optional[str] = None):
     """Print reasoning to user."""
     if json_output:
         click.echo(json.dumps(report, sort_keys=True, indent=2))
         return
 
-    table = Texttable(max_width=get_terminal_size().columns)
-    table.set_deco(Texttable.HEADER | Texttable.VLINES)
+    console = Console()
+    table = Table(
+        show_header=True,
+        header_style="bold green",
+        title=title,
+        box=box.MINIMAL_DOUBLE_HEAD,
+    )
 
-    header = set()
-    to_remove = set()
+    header = set()  # type: Set[str]
+    to_remove = set()  # type: Set[str]
     for item in report:
         header = header.union(set(item.keys()))
         to_remove = to_remove.union(
@@ -187,13 +220,15 @@ def _print_report(report: dict, json_output: bool = False):
     # Remove fields that can be an array - these are addition details that are supressed from the table output.
     header = header - to_remove
 
-    header = list(sorted(header))
-    table.set_cols_align([_TABLE_COLS_ALIGN.get(column, "l") for column in header])
-    table.header([item[0].upper() + item[1:].replace("_", " ") for item in header])
+    header_list = sorted(header)
+    for item in header_list:
+        table.add_column(
+            item.replace("_", " ").capitalize(), style="cyan", overflow="fold"
+        )
 
     for item in report:
         row = []
-        for column in header:
+        for column in header_list:
             entry = item.get(column, "-")
 
             if not bool(int(os.getenv("THAMOS_NO_EMOJI", 0))) and isinstance(
@@ -204,14 +239,40 @@ def _print_report(report: dict, json_output: bool = False):
             if isinstance(entry, list):
                 entry = ", ".join(entry)
 
+            if isinstance(entry, str) and entry.startswith(("https://", "http://")):
+                entry = f"[link {entry}]{entry}"
+
             row.append(entry)
 
-        table.add_row(row)
+        table.add_row(*row)
 
-    click.echo(table.draw())
+    console.print(table, justify="center")
 
 
-@click.group()
+class AliasedGroup(click.Group):
+    """Provide clever command aliases."""
+
+    def get_command(self, ctx, cmd_name):
+        """Get command to be executed based on the prefix.
+
+        https://click.palletsprojects.com/en/7.x/advanced/#command-aliases
+        """
+        if cmd_name == "logs":
+            # Address `thamos logs` vs `thamos log` confusion.
+            cmd_name = "log"
+
+        rv = click.Group.get_command(self, ctx, cmd_name)
+        if rv is not None:
+            return rv
+        matches = [x for x in self.list_commands(ctx) if x.startswith(cmd_name)]
+        if not matches:
+            return None
+        elif len(matches) == 1:
+            return click.Group.get_command(self, ctx, matches[0])
+        ctx.fail("Too many matches: %s" % ", ".join(sorted(matches)))
+
+
+@click.group(cls=AliasedGroup)
 @click.pass_context
 @click.option(
     "-v",
@@ -270,6 +331,13 @@ def _print_version(ctx, json_output: bool = False):
         api_url = configuration.api_url
     except NoProjectDirError as exc:
         _LOGGER.warning("Cannot obtain Thoth backend information: %s", str(exc))
+    except Exception as exc:
+        if _LOGGER.getEffectiveLevel() <= logging.DEBUG:
+            _log = _LOGGER.exception
+        else:
+            _log = _LOGGER.error
+
+        _log("Failed to obtain Thoth backend version: %s", str(exc))
 
     if json_output:
         click.echo(
@@ -291,6 +359,33 @@ def _print_version(ctx, json_output: bool = False):
     ctx.exit(0)
 
 
+@cli.command("install")
+@click.option(
+    "--dev/--no-dev",
+    is_flag=True,
+    default=False,
+    envvar="THAMOS_DEV",
+    show_default=True,
+    help="Consider or do not consider development dependencies during the installation process.",
+)
+@click.option(
+    "--runtime-environment",
+    "-r",
+    type=str,
+    default=None,
+    metavar="NAME",
+    envvar="THAMOS_RUNTIME_ENVIRONMENT",
+    help="Specify explicitly runtime environment for which the installation process should be done.",
+)
+def install(runtime_environment: str, dev: bool) -> None:
+    """Install dependencies as stated in Pipfile.lock or requirements.txt.
+
+    This command assumes requirements files are present and dependencies are already resolved.
+    If that's not the case, issue `thamos advise` before calling this.
+    """
+    thamos_install(runtime_environment_name=runtime_environment, dev=dev)
+
+
 @cli.command("advise")
 @click.option(
     "--debug",
@@ -317,6 +412,13 @@ def _print_version(ctx, json_output: bool = False):
     is_flag=True,
     help="Do not wait for analysis to finish, just submit it.",
 )
+@click.option(
+    "--no-user-stack",
+    is_flag=True,
+    help="Do not submit lock file with the request, this lock file is normally used as a base for "
+    "comparision to recommend a better stack than the one used.",
+    envvar="THAMOS_NO_USER_STACK",
+)
 @click.option(
     "--no-static-analysis",
     is_flag=True,
@@ -338,17 +440,10 @@ def _print_version(ctx, json_output: bool = False):
     type=str,
     default=None,
     metavar="NAME",
+    envvar="THAMOS_RUNTIME_ENVIRONMENT",
     help="Specify explicitly runtime environment to get recommendations for; "
     "defaults to the first entry in the configuration file.",
 )
-@click.option(
-    "--limit-latest-versions",
-    type=int,
-    default=None,
-    metavar="COUNT",
-    envvar="THAMOS_LIMIT_LATEST_VERSIONS",
-    help="Specify number of latest versions for each package to consider.",
-)
 @click.option(
     "--dev/--no-dev",
     envvar="THAMOS_DEV",
@@ -357,6 +452,23 @@ def _print_version(ctx, json_output: bool = False):
     show_default=True,
     help="Consider or do not consider development dependencies during the resolution.",
 )
+@click.option(
+    "--install",
+    envvar="THAMOS_INSTALL",
+    is_flag=True,
+    default=False,
+    show_default=True,
+    help="Install dependencies once the advise is done.",
+)
+@click.option(
+    "--write-advised-manifest-changes",
+    envvar="THAMOS_WRITE_ADVISED_MANIFEST_CHANGES",
+    type=str,
+    metavar="FILE",
+    default=None,
+    show_default=True,
+    help="Write advised manifest changes to a file.",
+)
 def advise(
     debug: bool = False,
     no_write: bool = False,
@@ -365,27 +477,38 @@ def advise(
     no_wait: bool = False,
     no_static_analysis: bool = False,
     json_output: bool = False,
-    limit_latest_versions: int = None,
     force: bool = False,
     dev: bool = False,
+    no_user_stack: bool = False,
+    install: bool = False,
+    write_advised_manifest_changes: Optional[str] = None,
 ):
     """Ask Thoth for recommendations on application stack."""
-    with workdir():
-        pipfile, pipfile_lock = _load_files(requirements_format=configuration.requirements_format)
+    if install and no_wait:
+        _LOGGER.error("Cannot install dependencies as --no-wait was provided")
+        sys.exit(1)
+    if install and no_write:
+        _LOGGER.error("Cannot install dependencies if lock files are not written")
+        sys.exit(1)
+
+    with cwd(configuration.get_overlays_directory(runtime_environment)):
+        if not dev and configuration.requirements_format == "pipenv":
+            _LOGGER.warning(
+                "Development dependencies will not be considered during the resolution process - see %s",
+                jl("no_dev"),
+            )
 
         # In CLI we always call to obtain only the best software stack (count is implicitly set to 1).
-        results = thoth_advise(
-            pipfile,
-            pipfile_lock,
+        results = thoth_advise_here(
             recommendation_type=recommendation_type,
             runtime_environment_name=runtime_environment,
             debug=debug,
             nowait=no_wait,
             force=force,
             source_type=ThothAdviserIntegrationEnum.CLI,
-            limit_latest_versions=limit_latest_versions,
             no_static_analysis=no_static_analysis,
             dev=dev,
+            no_user_stack=no_user_stack,
         )
 
         if not results:
@@ -399,33 +522,66 @@ def advise(
         result, error = results
         if error:
             if json_output:
-                print({"error": result["error_msg"]})
+                json.dump(result, sys.stdout, indent=2)
             else:
-                print(result["error_msg"])
+                stack_info = (result.get("report") or {}).get("stack_info")
+                if stack_info:
+                    _print_report(
+                        stack_info,
+                        json_output=json_output,
+                        title="Application stack guidance",
+                    )
+                print(
+                    result.get("error_msg")
+                    or "No error message was provided by the service."
+                )
+
             sys.exit(4)
 
         if not no_write:
+            if result["report"] and result["report"]["stack_info"]:
+                _print_report(
+                    result["report"]["stack_info"],
+                    json_output=json_output,
+                    title="Application stack guidance",
+                )
+
             # Print report of the best one - thus index zero.
             if result["report"] and result["report"]["products"]:
                 if result["report"]["products"][0]["justification"]:
-                    _print_header("Recommended stack report")
-                    _print_report(result["report"]["products"][0]["justification"], json_output=json_output)
+                    _print_report(
+                        result["report"]["products"][0]["justification"],
+                        json_output=json_output,
+                        title="Recommended stack report",
+                    )
                 else:
                     click.echo("No justification was made for the recommended stack")
 
-            if result["report"] and result["report"]["stack_info"]:
-                _print_header("Application stack guidance")
-                _print_report(result["report"]["stack_info"], json_output=json_output)
-
             pipfile = result["report"]["products"][0]["project"]["requirements"]
-            pipfile_lock = result["report"]["products"][0]["project"]["requirements_locked"]
+            pipfile_lock = result["report"]["products"][0]["project"][
+                "requirements_locked"
+            ]
             _write_configuration(
                 result["report"]["products"][0]["advised_runtime_environment"],
                 recommendation_type,
-                limit_latest_versions,
                 dev,
             )
-            _write_files(pipfile, pipfile_lock, configuration.requirements_format)
+            _write_files(pipfile, pipfile_lock, configuration.requirements_format)  # type: ignore
+
+            if write_advised_manifest_changes:
+                advised_manifest_changes = result["report"]["products"][0][
+                    "project"
+                ].get("advised_manifest_changes")
+                with open(
+                    write_advised_manifest_changes, "w"
+                ) as advised_manifest_changes_file:
+                    json.dump(
+                        advised_manifest_changes or {}, advised_manifest_changes_file
+                    )
+                    advised_manifest_changes_file.write("\n")
+
+            if install:
+                thamos_install(runtime_environment_name=runtime_environment, dev=dev)
         else:
             click.echo(json.dumps(result, indent=2))
 
@@ -447,6 +603,16 @@ def advise(
     is_flag=True,
     help="Do not wait for analysis to finish, just submit it.",
 )
+@click.option(
+    "--runtime-environment",
+    "-r",
+    type=str,
+    default=None,
+    metavar="NAME",
+    envvar="THAMOS_RUNTIME_ENVIRONMENT",
+    help="Specify explicitly runtime environment to get recommendations for; "
+    "defaults to the first entry in the configuration file.",
+)
 @click.option(
     "--force",
     is_flag=True,
@@ -460,11 +626,14 @@ def provenance_check(
     no_wait: bool = False,
     json_output: bool = False,
     force: bool = False,
+    runtime_environment: typing.Optional[str] = None,
 ):
     """Check provenance of installed packages."""
-    with workdir():
+    with cwd(configuration.get_overlays_directory(runtime_environment)):
         if configuration.requirements_format != "pipenv":
-            raise ValueError("Provenance checks are available only for requirements managed by Pipenv")
+            raise ValueError(
+                "Provenance checks are available only for requirements managed by Pipenv"
+            )
 
         pipfile, pipfile_lock = _load_files("pipenv")
         if not pipfile_lock:
@@ -483,9 +652,12 @@ def provenance_check(
             sys.exit(0)
 
         report, error = results
-        _print_report(report, json_output=json_output) if report else _LOGGER.info(
-            "Provenance check passed!"
-        )
+        if report:
+            _print_report(
+                report, json_output=json_output, title="Provenance check report"
+            )
+        else:
+            _LOGGER.info("Provenance check passed!")
 
         if error:
             sys.exit(5)
@@ -533,10 +705,20 @@ def status(analysis_id: str = None, output_format: str = None):
         status_dict = get_status(analysis_id)
 
     if not output_format or output_format == "table":
-        table = Texttable(max_width=get_terminal_size().columns)
-        table.set_deco(Texttable.VLINES)
-        table.add_rows(list(status_dict.items()), header=False)
-        output = table.draw()
+        table = Table()
+
+        for key in status_dict.keys():
+            table.add_column(
+                key.replace("_", " ").capitalize(),
+                style="cyan",
+                overflow="fold",
+            )
+
+        table.add_row(*status_dict.values())
+
+        console = Console()
+        console.print(table, justify="center")
+        return
     elif output_format == "json":
         output = json.dumps(status_dict, indent=2)
     elif output_format == "yaml":
@@ -547,6 +729,69 @@ def status(analysis_id: str = None, output_format: str = None):
     click.echo(output)
 
 
+@cli.command("list")
+def list_() -> None:
+    """List available runtime environments."""
+    with workdir(configuration.CONFIG_NAME):
+        environments = configuration.list_runtime_environments()
+
+    if not environments:
+        _LOGGER.error("No runtime environments found")
+        sys.exit(1)
+
+    for env in environments:
+        click.echo(env.get("name"))
+
+
+@cli.command("show")
+@click.option(
+    "--output-format",
+    "-o",
+    type=click.Choice(["json", "yaml"]),
+    default="yaml",
+    help="Specify output format for the status report.",
+    show_default=True,
+)
+@click.option(
+    "--runtime-environment",
+    "-r",
+    type=str,
+    default=None,
+    metavar="NAME",
+    envvar="THAMOS_RUNTIME_ENVIRONMENT",
+    help="Specify explicitly runtime environment to be shown.",
+)
+def show(output_format: str, runtime_environment: Optional[str] = None) -> None:
+    """Show configuration of available runtime environments."""
+    with workdir(configuration.CONFIG_NAME):
+        environments = configuration.list_runtime_environments()
+
+    if not runtime_environment:
+        if not environments:
+            _LOGGER.error("No runtime environments found")
+            sys.exit(1)
+
+    to_show = environments
+    if runtime_environment:
+        for env in environments:
+            if env.get("name") == runtime_environment:
+                to_show = env
+                break
+        else:
+            raise NoRuntimeEnvironmentError(
+                f"Runtime environment {runtime_environment!r} not found"
+            )
+
+    if output_format == "json":
+        output = json.dumps(to_show, indent=2)
+    elif output_format == "yaml":
+        output = yaml.safe_dump(to_show, default_flow_style=False)
+    else:
+        raise NotImplementedError(f"Unknown output format {output_format}")
+
+    click.echo(output)
+
+
 @cli.command("config")
 @click.option(
     "--no-interactive",
@@ -595,5 +840,65 @@ def config(no_interactive: bool = False, template: str = None):
         )
 
 
-if __name__ == "__main__":
-    cli()
+@cli.command("check")
+@click.option(
+    "--runtime-environment",
+    "-r",
+    type=str,
+    default=None,
+    metavar="NAME",
+    envvar="THAMOS_RUNTIME_ENVIRONMENT",
+    help="Specify explicitly runtime environment to check configuration file against.",
+)
+@click.option(
+    "--output-format",
+    "-o",
+    type=click.Choice(["json", "yaml", "table"]),
+    default="table",
+    help="Specify output format for the status report.",
+)
+def check(runtime_environment: Optional[str], output_format: str) -> None:
+    """Check configuration file and runtime environment."""
+    result = configuration.check(runtime_environment_name=runtime_environment)
+
+    if output_format == "yaml":
+        yaml.safe_dump(result, sys.stdout)
+    elif output_format == "json":
+        json.dump(result, sys.stdout, indent=2)
+        sys.stdout.write("\n")
+    elif output_format == "table":
+        table = Table()
+
+        header = set()
+        for item in result:
+            for key in item.keys():
+                header.add(key)
+
+        header_sorted = sorted(header)
+        for item in header_sorted:
+            table.add_column(
+                item.replace("_", " ").capitalize(),
+                style="cyan",
+                overflow="fold",
+            )
+
+        for item in result:
+            row = []
+            for key in header_sorted:
+                entry = item.get(key)
+                if not bool(int(os.getenv("THAMOS_NO_EMOJI", 0))) and isinstance(
+                    entry, str
+                ):
+                    entry = _EMOJI.get(entry, entry)
+
+                row.append(entry if entry is not None else "-")
+
+            table.add_row(*row)
+
+        console = Console()
+        console.print(table, justify="center")
+
+    sys.exit(1 if any(item.get("type") == "ERROR" for item in result) else 0)
+
+
+__name__ == "__main__" and cli()
