@@ -56,3 +56,11 @@ class ConfigurationError(ThamosException):
 
 class TimeoutError(ThamosException):
     """An exception raised if the API takes longer than timeout limit."""
+
+
+class ApiError(ThamosException):
+    """An exception raised if the API doesn't return expected status code."""
+
+
+class NoRequirementsFile(ThamosException):
+    """An exception raised when requirements file is not present."""
