@@ -29,7 +29,9 @@ from functools import wraps
 import pprint
 import json
 import urllib3
+import requests
 
+import micropipenv
 from termcolor import colored
 from yaspin import yaspin
 from yaspin.spinners import Spinners
@@ -37,6 +39,8 @@ from invectio import gather_library_usage
 from thoth.analyzer import run_command
 from thoth.python import Project
 from thoth.common import ThothAdviserIntegrationEnum
+from thoth.common import get_justification_link as jl
+from thoth.common import cwd
 
 from . import __version__ as thamos_version
 from .swagger_client.rest import ApiException
@@ -49,9 +53,14 @@ from .swagger_client import AdviseInput
 from .swagger_client import AdviseApi
 from .swagger_client import ImageAnalysisApi
 from .swagger_client import ProvenanceApi
+from .swagger_client.models import AnalysisResultResponse
 from .config import config as thoth_config
 from .exceptions import UnknownAnalysisType
 from .exceptions import TimeoutError
+from .exceptions import ApiError
+from .exceptions import NoRequirementsFile
+
+from typing import Callable, Any, Union, Dict
 
 urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
 
@@ -60,11 +69,17 @@ LAST_ANALYSIS_ID_FILE = ".thoth_last_analysis_id"
 _LOGGER = logging.getLogger(__name__)
 _RETRY_ON_ERROR_COUNT = int(os.getenv("THAMOS_RETRY_ON_ERROR_COUNT", 3))
 _RETRY_ON_ERROR_SLEEP = float(os.getenv("THAMOS_RETRY_ON_ERROR_SLEEP", 3.0))
-_THAMOS_TIMEOUT = int(os.getenv("THAMOS_TIMEOUT", 900))
+_THAMOS_TIMEOUT = int(os.getenv("THAMOS_TIMEOUT", 2000))
+_SOURCE = {
+    "adviser": "advise/python",
+    "provenance-checker": "provenance/python",
+    "package-extract": "analyze",
+}
 
 
 def with_api_client(func: typing.Callable):
     """Load configuration entries from Thoth configuration file."""
+    # noqa
     @wraps(func)
     def wrapper(*args, **kwargs):
         config = Configuration()
@@ -72,9 +87,7 @@ def with_api_client(func: typing.Callable):
         if not host:
             thoth_config.load_config()
             host = thoth_config.content.get("host") or config.host
-
         thoth_config.api_discovery(host)
-
         _LOGGER.debug("Using API: %s", thoth_config.api_url)
         config.host = thoth_config.api_url
         config.verify_ssl = thoth_config.tls_verify
@@ -93,8 +106,30 @@ def with_api_client(func: typing.Callable):
     return wrapper
 
 
-def _wait_for_analysis(status_func: callable, analysis_id: str) -> None:
+def is_analysis_ready(analysis_id: str) -> bool:
+    """Handle the the multiple response types available while asking for result of a anaylsis result."""
+    config = Configuration()
+    host = thoth_config.explicit_host
+    if not host:
+        thoth_config.load_config()
+        host = thoth_config.content.get("host") or config.host
+    source = analysis_id.rsplit("-", 1)[0]
+    source_url = _SOURCE.get(source)
+    response = requests.get(f"https://{host}/api/v1/{source_url}/{analysis_id}")
+    if response.status_code == 202:
+        return False
+    elif response.status_code in (200, 400):
+        # Return true if result is ready.
+        return True
+
+    raise ApiError(
+        f"Thoth Backend didn't respond with correct status code. Returned code - {response.status_code}"
+    )
+
+
+def _wait_for_analysis(status_func: Callable[..., Any], analysis_id: str) -> None:
     """Wait for ongoing analysis to finish."""
+    # noqa
     @contextmanager
     def _no_spinner():
         yield
@@ -103,7 +138,7 @@ def _wait_for_analysis(status_func: callable, analysis_id: str) -> None:
         yaspin,
         Spinners.clock,
         text=f"Waiting for response from Thoth (analysis: {analysis_id})...",
-    )
+    )  # type: Union[Callable[..., Any], partial[Any]]
     if _LOGGER.getEffectiveLevel() == logging.DEBUG or bool(
         int(os.getenv("THAMOS_NO_PROGRESSBAR", 1))
     ):
@@ -113,11 +148,13 @@ def _wait_for_analysis(status_func: callable, analysis_id: str) -> None:
     sleep_time = 0.5
     retries = 0
     with spinner():
-        sleep(2)  # TODO: remove once we fully run on Argo workflows
         start_time = monotonic()
         while True:
             if _THAMOS_TIMEOUT and monotonic() - start_time > _THAMOS_TIMEOUT:
-                raise TimeoutError(f"Thoth backend did not respond in time, timeout set to {_THAMOS_TIMEOUT}")
+                raise TimeoutError(
+                    f"Thoth backend did not respond in time, timeout set "
+                    f"to {_THAMOS_TIMEOUT} - see {jl('thamos_timeout')}"
+                )
             try:
                 response = status_func(analysis_id)
             except Exception as exc:
@@ -126,19 +163,25 @@ def _wait_for_analysis(status_func: callable, analysis_id: str) -> None:
 
                 retries += 1
                 _LOGGER.error("Failed to obtain status from Thoth: %s", str(exc))
-                _LOGGER.warning("Retrying in a few moments... (attempt %d/%d)", retries, _RETRY_ON_ERROR_COUNT)
+                _LOGGER.warning(
+                    "Retrying in a few moments... (attempt %d/%d)",
+                    retries,
+                    _RETRY_ON_ERROR_COUNT,
+                )
                 sleep(_RETRY_ON_ERROR_SLEEP)
                 continue
 
             retries = 0  # Reset counter as we obtained a valid response.
-            if response.status.finished_at is not None:
+            if response:
                 break
-            _LOGGER.debug(
-                "Waiting for %r to finish for %g seconds (state: %s)",
-                analysis_id,
-                sleep_time,
-                response.status.state,
-            )
+
+            if _LOGGER.getEffectiveLevel() <= logging.DEBUG:
+                _LOGGER.debug(
+                    "Waiting for %r to finish for %g seconds (state: %s)",
+                    analysis_id,
+                    sleep_time,
+                    (get_status(analysis_id) or {}).get("state", "N/A"),
+                )
 
             sleep(sleep_time)
             sleep_time = min(sleep_time * 2, 8)
@@ -172,14 +215,18 @@ def _get_last_analysis_id() -> str:
 
 
 def _retrieve_analysis_result(
-    retrieve_func: callable, analysis_id: str
-) -> typing.Optional[dict]:
+    retrieve_func: Callable[..., Any], analysis_id: str
+) -> Any:
     """Retrieve analysis result, raise error if analysis failed."""
     retries = 0
     while True:
         try:
             return retrieve_func(analysis_id)
         except ApiException as exc:
+            if exc.status == 400:
+                # Re-raise if the exception is bad request (e.g. analysis did not finish successfully).
+                raise
+
             _LOGGER.debug(
                 "Retrieved error response %s from server: %s", exc.status, exc.reason
             )
@@ -197,7 +244,11 @@ def _retrieve_analysis_result(
                 return None
 
             retries += 1
-            _LOGGER.warning("Retrying in a few moments... (attempt %d/%d)", retries, _RETRY_ON_ERROR_COUNT)
+            _LOGGER.warning(
+                "Retrying in a few moments... (attempt %d/%d)",
+                retries,
+                _RETRY_ON_ERROR_COUNT,
+            )
             sleep(_RETRY_ON_ERROR_SLEEP)
 
 
@@ -206,22 +257,24 @@ def _get_static_analysis() -> typing.Optional[dict]:
     # We are running in the root directory of project, use the root part for gathering static analysis.
     _LOGGER.info("Performing static analysis of sources to gather library usage")
     try:
-        library_usage = gather_library_usage(".", ignore_errors=True, without_standard_imports=True)
+        library_usage = gather_library_usage(
+            ".", ignore_errors=True, without_standard_imports=True
+        )
     except FileNotFoundError:
         _LOGGER.warning("No library usage was aggregated - no Python sources found")
         return None
 
-    report = {}
+    report = {}  # type: Dict[Any, Any]
     for file_record in library_usage["report"].values():
         for library, usage in file_record.items():
             # We could filter out some of the libraries which were used.
             if library not in report:
-                report[library] = []
+                report[library] = set()
 
-            report[library].extend(usage)
+            report[library].update(usage)
 
     return {
-        "report": report,
+        "report": {k: list(v) for k, v in report.items()},
         "version": library_usage["version"],
     }
 
@@ -229,14 +282,16 @@ def _get_static_analysis() -> typing.Optional[dict]:
 def _is_s2i() -> bool:
     """Check if we run in an OpenShift s2i build."""
     # This environment variable is used by OpenShift's s2i build process.
-    return "STI_SCRIPTS_PATH" is os.environ
+    return "STI_SCRIPTS_PATH" in os.environ
 
 
 def _get_origin() -> typing.Optional[str]:
     """Check git origin configured."""
     result = run_command("git config --get remote.origin.url", raise_on_error=False)
     if result.return_code != 0:
-        _LOGGER.debug("Failed to obtain information about git origin: %s", result.stderr)
+        _LOGGER.debug(
+            "Failed to obtain information about git origin: %s", result.stderr
+        )
         return None
 
     origin = result.stdout.strip()
@@ -247,6 +302,59 @@ def _get_origin() -> typing.Optional[str]:
     return None
 
 
+def advise_using_config(
+    pipfile: str,
+    pipfile_lock: str,
+    config: str = None,
+    *,
+    runtime_environment_name: str = None,
+    recommendation_type: str = None,
+    dev: bool = False,
+    no_static_analysis: bool = False,
+    no_user_stack: bool = False,
+    nowait: bool = False,
+    force: bool = False,
+    limit: int = None,
+    count: int = 1,
+    debug: bool = False,
+    origin: str = None,
+    github_event_type: typing.Optional[str] = None,
+    github_check_run_id: typing.Optional[int] = None,
+    github_installation_id: typing.Optional[int] = None,
+    github_base_repo_url: typing.Optional[str] = None,
+    source_type: typing.Optional[ThothAdviserIntegrationEnum] = None,
+) -> typing.Optional[typing.Tuple[typing.Dict[str, typing.Any], bool]]:
+    """Trigger advise, respecting the configuration file supplied directly as a string or as a file path."""
+    try:
+        thoth_config.load_config_from_file(config)
+    except (FileNotFoundError, IOError):
+        thoth_config.load_config_from_string(config)
+
+    return advise(
+        pipfile=pipfile,
+        pipfile_lock=pipfile_lock,
+        recommendation_type=recommendation_type,
+        runtime_environment=thoth_config.get_runtime_environment(
+            runtime_environment_name
+        ),
+        runtime_environment_name=runtime_environment_name,
+        dev=dev,
+        no_static_analysis=no_static_analysis,
+        no_user_stack=no_user_stack,
+        nowait=nowait,
+        force=force,
+        limit=limit,
+        count=count,
+        debug=debug,
+        origin=origin,
+        github_event_type=github_event_type,
+        github_check_run_id=github_check_run_id,
+        github_installation_id=github_installation_id,
+        github_base_repo_url=github_base_repo_url,
+        source_type=source_type,
+    )
+
+
 @with_api_client
 def advise(
     api_client: ApiClient,
@@ -256,9 +364,9 @@ def advise(
     *,
     runtime_environment: dict = None,
     runtime_environment_name: str = None,
-    limit_latest_versions: int = None,
     dev: bool = False,
     no_static_analysis: bool = False,
+    no_user_stack: bool = False,
     nowait: bool = False,
     force: bool = False,
     limit: int = None,
@@ -275,27 +383,20 @@ def advise(
     if not pipfile:
         raise ValueError("No Pipfile content provided for advises")
 
-    if runtime_environment and runtime_environment_name:
-        raise ValueError(
-            "Cannot use runtime_environment and runtime_environment_name at the same time"
-        )
-
     if runtime_environment is None:
         runtime_environment = (
             thoth_config.get_runtime_environment(runtime_environment_name) or dict()
         )
 
-    # We use the explicit one if provided at the end.
-    if limit_latest_versions is None:
-        priority = (
-            runtime_environment.pop("limit_latest_versions", None),
-            thoth_config.content.get("limit_latest_versions", None),
-            None,
+    if runtime_environment_name is None and runtime_environment:
+        runtime_environment_name = runtime_environment.get("name")
+
+    if (runtime_environment and runtime_environment_name) and runtime_environment.get(
+        "name"
+    ) != runtime_environment_name:
+        raise ValueError(
+            "Runtime environment name supplied does not match with the one stated in the config file"
         )
-        try:
-            limit_latest_versions = next(filter(bool, priority))
-        except StopIteration:
-            limit_latest_versions = None
 
     if recommendation_type is None:
         priority = (
@@ -305,20 +406,27 @@ def advise(
         )
         recommendation_type = next(filter(bool, priority))
 
+    if no_user_stack and pipfile_lock:
+        _LOGGER.warning(
+            "The user stack found in the lock file will not be supplied as requested"
+        )
+        pipfile_lock = ""
+
     library_usage = None
     if not no_static_analysis:
         library_usage = _get_static_analysis()
-        _LOGGER.debug("Library usage:%s", "\n" + json.dumps(library_usage, indent=2) if library_usage else None)
+        _LOGGER.debug(
+            "Library usage:%s",
+            "\n" + json.dumps(library_usage, indent=2) if library_usage else None,
+        )
 
     stack = PythonStack(requirements=pipfile, requirements_lock=pipfile_lock or "")
 
     if runtime_environment:
         # Override recommendation type specified explicitly in the runtime environment entry.
         runtime_environment.pop("recommendation_type", None)
-        # Override latest versions limit specified explicitly in the runtime environment entry.
-        runtime_environment.pop("limit_latest_versions", None)
 
-        runtime_environment = RuntimeEnvironment(**runtime_environment)
+        runtime_environment = RuntimeEnvironment(**runtime_environment)  # type: ignore
 
     advise_input = AdviseInput(
         stack, runtime_environment=runtime_environment, library_usage=library_usage
@@ -335,10 +443,10 @@ def advise(
         "recommendation_type": recommendation_type,
         "debug": debug,
         "force": force,
-        "source_type": source_type.name,
+        "source_type": source_type.name.lower() if source_type is not None else None,
         "origin": _get_origin(),
         "dev": dev,
-    }
+    }  # type: Dict[str, Any]
 
     if limit is not None:
         parameters["limit"] = limit
@@ -346,9 +454,6 @@ def advise(
     if count is not None:
         parameters["count"] = count
 
-    if limit_latest_versions is not None:
-        parameters["limit_latest_versions"] = limit_latest_versions
-
     if origin is not None:
         parameters["origin"] = origin
 
@@ -379,8 +484,8 @@ def advise(
 
     if nowait:
         return response.analysis_id
-
-    _wait_for_analysis(api_instance.get_advise_python_status, response.analysis_id)
+    # We call custom status function for advise until swagger client supports mulitple response codes.
+    _wait_for_analysis(is_analysis_ready, response.analysis_id)
     _LOGGER.debug("Retrieving adviser result for %r", response.analysis_id)
     response = _retrieve_analysis_result(
         api_instance.get_advise_python, response.analysis_id
@@ -398,9 +503,9 @@ def advise_here(
     *,
     runtime_environment: dict = None,
     runtime_environment_name: typing.Optional[str] = None,
-    limit_latest_versions: typing.Optional[int] = None,
     dev: bool = False,
     no_static_analysis: bool = False,
+    no_user_stack: bool = False,
     nowait: bool = False,
     force: bool = False,
     limit: typing.Optional[int] = None,
@@ -416,11 +521,42 @@ def advise_here(
     """Run advise in current directory, requires no arguments."""
     requirements_format = thoth_config.requirements_format
     if requirements_format == "pipenv":
-        project = Project.from_files(without_pipfile_lock=not os.path.exists("Pipfile.lock"))
+        _LOGGER.info(
+            "Using Pipenv files to manage dependencies located in %r", os.getcwd()
+        )
+        pipfile_lock_exists = os.path.exists("Pipfile.lock")
+
+        if pipfile_lock_exists:
+            _LOGGER.info(
+                "Submitting Pipfile.lock as a base for user's stack scoring - see %s",
+                jl("user_stack"),
+            )
+
+        project = Project.from_files(
+            without_pipfile_lock=not os.path.exists("Pipfile.lock")
+        )
+
+        if (
+            pipfile_lock_exists
+            and project.pipfile_lock.meta.hash["sha256"]
+            != project.pipfile.hash()["sha256"]
+        ):
+            _LOGGER.error(
+                "Pipfile hash stated in Pipfile.lock %r does not correspond to Pipfile hash %r - was Pipfile "
+                "adjusted? This error is not critical.",
+                project.pipfile_lock.meta.hash["sha256"][:6],
+                project.pipfile.hash()["sha256"][:6],
+            )
     elif requirements_format in ("pip", "pip-tools", "pip-compile"):
+        _LOGGER.info(
+            "Using requirements.txt file to manage dependencies located in %r",
+            os.getcwd(),
+        )
         project = Project.from_pip_compile_files(allow_without_lock=True)
     else:
-        raise ValueError(f"Unknown configuration option for requirements format: {requirements_format!r}")
+        raise ValueError(
+            f"Unknown configuration option for requirements format: {requirements_format!r}"
+        )
 
     pipfile = project.pipfile.to_string()
     pipfile_lock_str = project.pipfile_lock.to_string() if project.pipfile_lock else ""
@@ -431,16 +567,16 @@ def advise_here(
         recommendation_type=recommendation_type,
         runtime_environment=runtime_environment,
         runtime_environment_name=runtime_environment_name,
-        limit_latest_versions=limit_latest_versions,
         dev=dev,
         no_static_analysis=no_static_analysis,
+        no_user_stack=no_user_stack,
         nowait=nowait,
         force=force,
         limit=limit,
         count=count,
         debug=debug,
         origin=origin,
-        source_type=source_type.name,
+        source_type=source_type,
         github_event_type=github_event_type,
         github_check_run_id=github_check_run_id,
         github_installation_id=github_installation_id,
@@ -479,7 +615,7 @@ def provenance_check(
     if nowait:
         return response.analysis_id
 
-    _wait_for_analysis(api_instance.get_provenance_python_status, response.analysis_id)
+    _wait_for_analysis(is_analysis_ready, response.analysis_id)
     _LOGGER.debug("Retrieving provenance check result for %r", response.analysis_id)
     response = _retrieve_analysis_result(
         api_instance.get_provenance_python, response.analysis_id
@@ -528,7 +664,7 @@ def image_analysis(
     nowait: bool = False,
     force: bool = False,
     debug: bool = False,
-) -> typing.Union[typing.Dict, str]:
+) -> Union[Dict, str, None]:
     """Submit an image for analysis to Thoth."""
     if not image:
         raise ValueError("No image provided")
@@ -559,7 +695,7 @@ def image_analysis(
     if nowait:
         return response.analysis_id
 
-    _wait_for_analysis(api_instance.get_analyze_status, response.analysis_id)
+    _wait_for_analysis(is_analysis_ready, response.analysis_id)
     _LOGGER.debug(
         "Retrieving image analysis result result for %r", response.analysis_id
     )
@@ -579,9 +715,13 @@ def build_analysis(
     output_image: str,
     *,
     environment_type: str,
-    registry_user: str = None,
-    registry_password: str = None,
-    registry_verify_tls: bool = True,
+    base_registry_user: typing.Optional[str] = None,
+    base_registry_password: typing.Optional[str] = None,
+    base_registry_verify_tls: bool = True,
+    output_registry_user: typing.Optional[str] = None,
+    output_registry_password: typing.Optional[str] = None,
+    output_registry_verify_tls: bool = True,
+    origin: typing.Optional[str] = None,
     nowait: bool = False,
     force: bool = False,
     debug: bool = False,
@@ -596,28 +736,29 @@ def build_analysis(
     else:
         raise ValueError("No build info provided")
 
+    params = dict(
+        body=build_detail,
+        base_registry_user=base_registry_user,
+        base_registry_password=base_registry_password,
+        base_registry_verify_tls=base_registry_verify_tls,
+        output_registry_user=output_registry_user,
+        output_registry_password=output_registry_password,
+        output_registry_verify_tls=output_registry_verify_tls,
+        environment_type=environment_type,
+        origin=origin,
+        debug=debug,
+        force=force,
+    )
+
+    # Swagger client handles None in a different way - we need to explicitly avoid passing
+    # values that are not present.
+    params = {k: v for k, v in params.items() if v is not None}
+
     api_instance = BuildAnalysisApi(api_client)
-    if registry_user or registry_password:
-        # Swagger client handles None in a different way - we need to explicitly avoid passing
-        # registry user and registry password if they are not set.
-        response = api_instance.post_build(
-            body=build_detail,
-            debug=debug,
-            registry_user=registry_user,
-            registry_password=registry_password,
-            registry_verify_tls=registry_verify_tls,
-            force=force,
-            environment_type=environment_type,
-        )
-    else:
-        response = api_instance.post_build(
-            body=build_detail,
-            debug=debug,
-            registry_verify_tls=registry_verify_tls,
-            force=force,
-        )
+    response = api_instance.post_build(**params)
 
     _LOGGER.info("Successfully submitted build analysis to %r", thoth_config.api_url)
+    _LOGGER.debug("Build analysis parameters: %r", params)
     if nowait:
         return response
 
@@ -634,14 +775,16 @@ def get_log(api_client: ApiClient, analysis_id: str = None):
         analysis_id = _get_last_analysis_id()
 
     if analysis_id.startswith("package-extract-"):
-        api_instance = ImageAnalysisApi(api_client)
-        method = api_instance.get_analyze_log
+        api_instance = ImageAnalysisApi(
+            api_client
+        )  # type: Union[ImageAnalysisApi, ProvenanceApi, AdviseApi]
+        method = api_instance.get_analyze_log  # type: ignore
     elif analysis_id.startswith("provenance-checker-"):
         api_instance = ProvenanceApi(api_client)
-        method = api_instance.get_provenance_python_log
+        method = api_instance.get_provenance_python_log  # type: ignore
     elif analysis_id.startswith("adviser-"):
         api_instance = AdviseApi(api_client)
-        method = api_instance.get_advise_python_log
+        method = api_instance.get_advise_python_log  # type: ignore
     else:
         raise UnknownAnalysisType(
             "Cannot determine analysis type from identifier: %r", analysis_id
@@ -658,9 +801,9 @@ def get_log(api_client: ApiClient, analysis_id: str = None):
             content = json.loads(line)
             if not int(os.getenv("THAMOS_NO_EMOJI", 0)):
                 if content["levelname"] == "DEBUG":
-                    message = colored(content["message"], "cyan")
-                elif content["levelname"] == "INFO":
                     message = colored(content["message"], "green")
+                elif content["levelname"] == "INFO":
+                    message = colored(content["message"], "cyan")
                 elif content["levelname"] == "WARNING":
                     message = colored(content["message"], "yellow", attrs=["bold"])
                 elif content["levelname"] in ("ERROR", "CRITICAL"):
@@ -668,9 +811,16 @@ def get_log(api_client: ApiClient, analysis_id: str = None):
                 else:
                     message = content["message"]
 
-                result += "{} {}: {}\n".format(content["asctime"], content["levelname"], message)
+                result += "{} {:<27} {}: {}\n".format(
+                    content["asctime"], content["name"], content["levelname"], message
+                )
             else:
-                result += "{} {}: {}\n".format(content["asctime"], content["levelname"], content["message"])
+                result += "{} {:<27} {}: {}\n".format(
+                    content["asctime"],
+                    content["name"],
+                    content["levelname"],
+                    content["message"],
+                )
         except Exception:
             # If the content parsed does not carry logger information or has not relevant
             # entries, log the original message.
@@ -690,14 +840,16 @@ def get_status(api_client: ApiClient, analysis_id: str = None):
         analysis_id = _get_last_analysis_id()
 
     if analysis_id.startswith("package-extract-"):
-        api_instance = ImageAnalysisApi(api_client)
-        method = api_instance.get_analyze_status
+        api_instance = ImageAnalysisApi(
+            api_client
+        )  # type: Union[ImageAnalysisApi, ProvenanceApi, AdviseApi]
+        method = api_instance.get_analyze_status  # type: ignore
     elif analysis_id.startswith("provenance-checker-"):
         api_instance = ProvenanceApi(api_client)
-        method = api_instance.get_provenance_python_status
+        method = api_instance.get_provenance_python_status  # type: ignore
     elif analysis_id.startswith("adviser-"):
         api_instance = AdviseApi(api_client)
-        method = api_instance.get_advise_python_status
+        method = api_instance.get_advise_python_status  # type: ignore
     else:
         raise UnknownAnalysisType(
             "Cannot determine analysis type from identifier: %r", analysis_id
@@ -710,21 +862,79 @@ def get_status(api_client: ApiClient, analysis_id: str = None):
 def get_analysis_results(api_client: ApiClient, analysis_id: str):
     """Get the analysis result from a given id."""
     if analysis_id.startswith("package-extract-"):
-        api_instance = ImageAnalysisApi(api_client)
-        method = api_instance.get_analyze
-        response = _retrieve_analysis_result(method, analysis_id)
+        api_instance = ImageAnalysisApi(
+            api_client
+        )  # type: Union[ImageAnalysisApi, ProvenanceApi, AdviseApi]
+        method = api_instance.get_analyze  # type: ignore
+        response = _retrieve_analysis_result(
+            method, analysis_id
+        )  # type: AnalysisResultResponse
         return response.result
     elif analysis_id.startswith("provenance-checker-"):
         api_instance = ProvenanceApi(api_client)
-        method = api_instance.get_provenance_python
+        method = api_instance.get_provenance_python  # type: ignore
         response = _retrieve_analysis_result(method, analysis_id)
         return response.result["report"], response.result["error"]
     elif analysis_id.startswith("adviser-"):
         api_instance = AdviseApi(api_client)
-        method = api_instance.get_advise_python
+        method = api_instance.get_advise_python  # type: ignore
         response = _retrieve_analysis_result(method, analysis_id)
         return response.result, response.result["error"]
     else:
         raise UnknownAnalysisType(
             "Cannot determine analysis type from identifier: %r", analysis_id
         )
+
+
+def install_using_config(
+    config: str,
+    runtime_environment_name: typing.Optional[str] = None,
+    dev: bool = False,
+) -> None:
+    """Perform installation given the configuration supplied."""
+    try:
+        thoth_config.load_config_from_file(config)
+    except (FileNotFoundError, IOError):
+        thoth_config.load_config_from_string(config)
+
+    install(runtime_environment_name=runtime_environment_name, dev=dev)
+
+
+def install(
+    runtime_environment_name: typing.Optional[str] = None, dev: bool = False
+) -> None:
+    """Perform installation of packages for the given runtime environment.
+
+    If the runtime environment is not specified, the first environment stated in the configuration is used.
+    """
+    method = (
+        "pipenv" if thoth_config.requirements_format == "pipenv" else "requirements"
+    )
+
+    if not dev and method == "pipenv":
+        _LOGGER.warning(
+            "Development dependencies will not be installed - see %s", jl("no_dev")
+        )
+
+    with cwd(thoth_config.get_overlays_directory(runtime_environment_name)):
+        if method == "pipenv":
+            if not os.path.isfile("Pipfile.lock"):
+                raise NoRequirementsFile(
+                    f"No Pipfile.lock found in {os.getcwd()!r} needed to install dependencies"
+                )
+            if not os.path.isfile("Pipfile"):  # Required for computing digests.
+                raise NoRequirementsFile(
+                    f"No Pipfile found in {os.getcwd()!r} needed for the installation process"
+                )
+        else:
+            if not os.path.isfile("requirements.txt"):
+                raise NoRequirementsFile(
+                    f"No requirements.txt file found in {os.getcwd()!r} needed to install dependencies"
+                )
+
+        _LOGGER.info(
+            "Using %r installation method to install dependencies stated in %r",
+            method,
+            os.getcwd(),
+        )
+        micropipenv.install(method=method, deploy=True, dev=dev)
